#!/usr/bin/perl
use strict;
use lib('/home/bmajoros/genomics/browser',
	'/home/bmajoros/genomics/perl',
	'/home/bmajoros/perlib',
	'.');
use GeneScript::Color;
use GeneScript::Signal;
use GeneScript::Canvas;
use GeneScript::PostscriptPage;
use GeneScript::ColorLegend;
use GffTranscriptReader;

my $MARGIN=0.1;

my $usage="$0 <matrix.txt> <promer.coords> <*.gff> <*.gff> <outfile.ps>";
die "$usage\n" unless @ARGV==5;
my ($infile,$promerFile,$gffFile1,$gffFile2,$outfile)=@ARGV;

my @colors=($Color::red,$Color::blue,$Color::green,$Color::gray,
	    $Color::brown,$Color::purple,$Color::lightGreen,
	    $Color::lightBlue,$Color::hotPink,$Color::gold,
	    $Color::darkGray,$Color::yellow,$Color::orange);

my $page=new GeneScript::PostscriptPage;
my $canvas=new GeneScript::Canvas;
$canvas->setLineWidth(0.001);
$canvas->drawText($outfile,0.1,0.92,0.018*length($outfile),0.03);

# LOAD MATRIX ****************************************************************
my ($minX,$maxX,$minY,$maxY,@cols,%Ys);
open(IN,$infile) || die "can't open $infile\n";
while(<IN>)
  {
    if(/^(\d+)\s+:\s+(\d+)\s+\((\d+):(\d+)\s+-\s+(\d+):(\d+)\)\s+POS=(\S+)\s+(\S+)/)
      {
	my ($col,$size,$from,$fromPos,$to,$toPos,$pos,$ys)=($1,$2,$3,$4,$5,$6,$7,$8);
	push @cols,[$col,$size,$from,$to,$pos,$ys];
	if(!defined($minY) || $from<$minY) {$minY=$from}
	if(!defined($maxY) || $to>$maxY) {$maxY=$to}
	if(!defined($minX) || $col<$minX) {$minX=$col}
	if(!defined($maxX) || $col>$maxX) {$maxX=$col}
	$Ys{$from}=$fromPos;
	$Ys{$to}=$toPos;
      }
  }
close(IN);
my @Ys;
my @domain=sort {$a <=> $b} keys %Ys;
foreach my $d (@domain) {push @Ys,[$d,$Ys{$d}]}

# DRAW MATRIX **************************************************************
my $xRange=$maxX-$minX;
my $yRange=$maxY-$minY;
my $xWidth=(1-2*$MARGIN)/$xRange;
my $yWidth=(1-2*$MARGIN)/$yRange;
foreach my $col (@cols)
  {
    my ($x,$height,$fromY,$toY,$pos,$ys)=@$col;
    chop $ys;
    my @ys=split/,/,$ys;
    foreach my $y (@ys) {$canvas->drawRect(mapX($x),mapY($y),$xWidth,$yWidth)}
#    for(my $y=$fromY ; $y<=$toY ; ++$y)
#      {
#	$canvas->drawRect(mapX($x),mapY($y),$xWidth,$yWidth);
#      }
  }

# LOAD AND DRAW PROMER HITS *************************************************
my $INC=0.002;
my $y=$MARGIN-$INC;
my $x=$MARGIN-$INC;
open(IN,$promerFile) || die "can't open $promerFile\n";
while(<IN>)
  {
    if(/^\s+(\S+)\s+(\S+)\s+\|\s+(\S+)\s+(\S+)\s+\|\s+\S+\s+\S+\s+\|\s+(\S+)\s+(\S+)/)
      {
	my ($from1,$to1,$from2,$to2,$id,$sim)=($1,$2,$3,$4,$5,$6);
	if($from1>$to1) {($from1,$to1)=($to1,$from1)}
	if($from2>$to2) {($from2,$to2)=($to2,$from2)}
	if($id<50 || $sim<65) {next}

	my $begin=$MARGIN+(1-2*$MARGIN)*findCellX($from1)/$xRange;
	my $end=$MARGIN+(1-2*$MARGIN)*findCellX($to1)/$xRange;
	$canvas->drawLine($begin,$y,$end,$y);
	$y-=$INC;

	my $beginY=$MARGIN+(1-2*$MARGIN)*findCellY($from2)/$yRange;
	my $endY=$MARGIN+(1-2*$MARGIN)*findCellY($to2)/$yRange;
	$canvas->drawLine($x,$beginY,$x,$endY);
	$x-=$INC;

	#$canvas->drawLine($begin,$beginY,$end,$endY);
      }
  }
close(IN);

# LOAD GFF AND DRAW GENE MODELS **********************************************
my $reader=new GffTranscriptReader();
my $transcripts1=$reader->loadGFF($gffFile1);
my $transcript1=$transcripts1->[0];
my $numExons=$transcript1->numExons();
for(my $i=0 ; $i<$numExons ; ++$i)
  {
    my $exon=$transcript1->getIthExon($i);
    my $begin=findCellX($exon->{begin});
    my $end=findCellX($exon->{end});
    $canvas->drawRect(mapX($begin),0.03,mapX($end)-mapX($begin),0.03);

    my $db=mapX($begin);
    my $de=mapX($end);
    my $dif=$de-$db;
    print "$exon->{begin} $exon->{end} $begin $end $db 0.03 $dif 0.03\n";
  }
my $transcripts2=$reader->loadGFF($gffFile2);
my $transcript2=$transcripts2->[0];
my $numExons=$transcript2->numExons();
for(my $i=0 ; $i<$numExons ; ++$i)
  {
    my $exon=$transcript2->getIthExon($i);
    my $begin=findCellY($exon->{begin});
    my $end=findCellY($exon->{end});
    $canvas->drawRect(0.03,mapY($begin),0.03,mapY($end)-mapY($begin));
  }

# ATTACH CANVAS AND EJECT PAGE ***********************************************
$page->attach($canvas,0,0,1,1);
$page->changeMargins(0,0);
my $ps=$page->toPostscript();
open(PS,">$outfile") || die;
print PS $ps;
close(PS);
#-----------------------------------------------------------------------
sub mapX
  {
    my ($x)=@_;
    return $MARGIN+(1-2*$MARGIN)*$x/$xRange;
  }
sub mapY
  {
    my ($y)=@_;
    return $MARGIN+(1-2*$MARGIN)*$y/$yRange;
  }
#-----------------------------------------------------------------------
sub findCellX
  {
    my ($x)=@_;
    foreach my $col (@cols)
      {
	my ($index,$height,$fromY,$toY,$pos)=@$col;
	if($x<$pos) {return $index}
      }

    my $firstCol=$cols[0];
    my $lastCol=$cols[@cols-1];
    if($x<=$firstCol->[4]) {return 0}
    if($x>=$lastCol->[4]) {return $lastCol->[0]}
    die;
  }
#-----------------------------------------------------------------------
sub findCellY
  {
    my ($x)=@_;
    foreach my $entry (@Ys)
      {
	my ($index,$pos)=@$entry;
	if($x<$pos) {return $index}
      }
    my $firstRow=$Ys[0];
    my $lastRow=$Ys[@Ys-1];
    if($x<=$firstRow->[1]) {return 0}
    if($x>=$lastRow->[1]) {return $lastRow->[0]}
    die;
  }
#-----------------------------------------------------------------------
#-----------------------------------------------------------------------
#-----------------------------------------------------------------------
#-----------------------------------------------------------------------
#-----------------------------------------------------------------------
#-----------------------------------------------------------------------
#-----------------------------------------------------------------------
