/****************************************************************
 oasis/oasis
 bmajoros@tigr.org

 Copyright (c)2004, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_oasis_h
#define INCL_oasis_h

#include "tigr++/TigrCommandLine.H"
#include "tigr++/DnaAlphabet.H"
#include "tigr++/TigrConfigFile.H"
#include "tigr++/TigrMap.H"
#include "tigr++/TigrStack.H"
#include "tigr++/Constants.H"
#include "tigr++/TigrTime.H"
#include "tigr++/TigrArray1D.H"
#include "tigr++/BinomialDistribution.H"
#include "tigr++/TigrProteinTrans.H"
#include "tigr++/TigrRegex.H"
#include "tigr++/TigrProgress.H"
#include "tigr++/AminoAlphabet.H"
#include "tigr++/TigrBitSet.H"
#include "tigrscan/TigrScan.H"
#include "tigrscan/SignalSensor.H"
#include "tigrscan/SignalQueue.H"
#include "tigrscan/SignalTypeProperties.H"
#include "tigrscan/NoncodingQueue.H"
#include "tigrscan/IntronQueue.H"
#include "tigrscan/EmpiricalDistribution.H"
#include "tigrscan/GeometricDistribution.H"
#include "tigrscan/Transitions.H"
#include "tigrscan/IsochoreFile.H"
#include "tigrscan/GffReader.H"
#include "tigrscan/TopologyLoader.H"
#include "tigrscan/Edge.H"
#include "tigrscan/GarbageCollector.H"
#include "tigrscan/ParseGraph.H"
#include "tigr++/NormalDistribution.H"
#include "tigr++/PoissonDistribution.H"
#include "tigr++/ExtremeValueDistribution.H"
#include "tigr++/DiscreteDistr.H"
#include "alignment/ApproximateAlignment.H"
#include "PromerSignalMap.H"
#include "OasisMatrix.H"
#include "OasisEdgeFactory.H"
#include "OasisEdge.H"

#define REPORT_MEMORY_USAGE
typedef float FLOAT_OR_DOUBLE;


class Oasis
{
  float nucPmatch, aminoPmatch, nucPmismatch, aminoPmismatch; // logs
  TigrString referenceSpecies, querySpecies;
  TigrRegex isoRegex;
  OasisMatrix *matrix;
  ParseGraph *referenceGraph;
  ParseGraph *queryGraph;
  Sequence *referenceSeq, *querySeq;
  TigrString *referenceSeqStr, *querySeqStr;
  OasisEdgeFactory ef;
  SubstitutionMatrix<FLOAT_OR_DOUBLE> *nucMatrix, *aminoMatrix;
  float gapExtendPenalty;
  int numRightTermini;
  bool swapPromerRefQuery; // whether to swap reference & query in promer
  TigrString matrixOutFile; // for monitoring the size of the matrix, etc.
  ApproximateAlignment<FLOAT_OR_DOUBLE> *nucAlignment;
  TigrVector<ApproximateAlignment<FLOAT_OR_DOUBLE>*> aminoAlignments;
  TigrBitSet tempAlignmentSet;
  float gcContent;

  int reductionLevel;//how aggressively we try to reduce the DP matrix (0-4)
  int signalTolerance;//bp tolerance for exact mapping to individual signal

  /* when a signal does not fall within a promer match,
     we use the nearest match to constrain the extent
     of the column in that direction; this parameter
     expresses how far along the match we go before
     imposing the constraint; it is a percentage of the
     match length: */
  float offMatchPortion;

  /* When a signal is mapped by promer to a specific location in the other
     genome but there is not an identical signal there, we collect all 
     signals within +/- this number of base pairs to that point in the 
     other genome and allocate cells in the column for those signals: */
  int nonexactSignalTolerance;
  
  /* bp to extend match to cover signals that occur
     just off the edge of the match.  these can then
     be mapped in an exact or nonexact manner: */
  int matchEdgeTolerance; 
  
  int minIdent, minSim, maxStop; // these are percentages!

  TigrStringMap<char> *stopCodons;
  
  void processConfigFile(TigrScan &);
  SubstitutionMatrix<FLOAT_OR_DOUBLE> *loadSubstMatrix(const TigrString &
						       filename,
						       bool amino);
  void allocateMatrix(const TigrString &deltaFile,int signalTolerance, 
		      int matchEdgeTolerance,int minIdent,int minSim,
		      int maxStop,ParseGraph &referenceGraph,
		      ParseGraph &queryGraph,float offMatchPortion);
  int getMinRowOnLeft(TigrSet<Edge*> &edgesIn,TigrArray1D<int> &minRows,
		      int numRows);
  int getMaxRowOnRight(TigrSet<Edge*> &edgesOut,TigrArray1D<int> &maxRows,
		       int numRows);
  void evaluateMatrix();
  void evaluatePredecessor(Cell &thisCell,Cell &predCell,
			   Edge *queryEdge,Edge *referenceEdge,
			   TigrArray1D<CellLink> &);
  void installLinks(TigrArray1D<CellLink> &from,TigrArray1D<CellLink> &to);
  int countActiveLinks(TigrArray1D<CellLink> &);
  void evaluateReferencePred(Cell &cell,Cell &pred,
			     TigrArray1D<CellLink> &links,
			     FLOAT_OR_DOUBLE queryEdgeAndSignalScore,
			     Edge *referenceEdge,
			     Edge *queryEdge,int queryPhase);
  void evaluateAlignment(Cell &cell,Cell &pred,
			 FLOAT_OR_DOUBLE edgeAndSignalScores,
			 TigrArray1D<CellLink> &links,int oldQueryPhase,
			 int newQueryPhase,int oldReferencePhase,
			 Edge *queryEdge,Edge *referenceEdge);
  void traceBack(TigrStack<SignalPtr> &referenceModel,
		 TigrStack<SignalPtr> &queryModel);
  int traceBackPhase(int phase,Signal &signal,Signal &pred);
  void output(TigrStack<SignalPtr> &model,const TigrString &filename,
	      TigrScan &,const TigrString &species);
  void debugGFF(TigrVector<TigrGffTranscript*> &qTranscripts,
		TigrVector<TigrGffTranscript*> &rTranscripts,
		ParseGraph &referenceGraph,ParseGraph &queryGraph,
		TigrScan &qTigrScan,TigrScan &rTigrScan,
		Sequence &qSeq,const TigrString &qSeqStr,
		Sequence &rSeq,const TigrString &rSeqStr);
  void debugGFF(TigrVector<SignalPtr> &path,TigrVector<BOOL> &found,
		const TigrString &seq);
  int getPhase(Signal &);
  void computePathScore(TigrStack<SignalPtr> &path);
  void loadApproxAminoAligns(const TigrString &aminoAppalFile,
			     const SubstitutionMatrix<FLOAT_OR_DOUBLE> &,
			     const Sequence &seq1,const char *seqStr1,
			     const Sequence &seq2,const char *seqStr2);
  ApproxAminoAlignment<FLOAT_OR_DOUBLE> *findAminoAlignment();
  void registerAminoAlignments();
  void registerAminoAlignment(int alignmentId,int beginX,int endX,
			      int beginY,int endY,ParseGraph &);
  void registerRows(Column &,int alignmentId,int beginY,int endY);
  double getAminoAlignmentScore(Cell &predCell,Cell &thisCell,
				Edge &refEdge,Edge &queryEdge,
				ApproximateAlignment<FLOAT_OR_DOUBLE> 
				*&chosenAlignment,bool debugging=false);
  void debugPathPair(TigrVector<SignalPtr> &referencePath,
		     TigrVector<SignalPtr> &queryPath);

public:
  Oasis();
  int main(int argc,char *argv[]);
};


#endif
