#!/usr/bin/perl
use strict;

# PROCESS THE COMMAND LINE
my $usage="$0 <*.delta>";
die "$usage\n" unless @ARGV==1;
my ($deltaFile)=@ARGV;

# LOAD THE DELTA FILE
my %deltaFile;
open(IN,$deltaFile) || die "Can't open $deltaFile\n";
<IN>;<IN>;<IN>;
while(<IN>)
  {
    my $header=$_;
    my @fields=split/\s+/,$_;
    die "error parsing line in $deltaFile: $_\n" unless @fields==7;
    my ($queryBegin,$queryEnd,$refBegin,$refEnd,$numIdent,$numSim,$numStops)=
      @fields;
    --$refBegin; --$queryBegin;
    my @deltas;
    while(1)
      {
	my $delta=0+<IN>;
	last unless $delta;
	push @deltas,$delta;
      }
    my $queryLen=$queryEnd-$queryBegin;
    my $refLen=$refEnd-$refBegin;
    next unless $queryLen>0 && $refLen>0;
    print;
    my $appal=promerToAppal(\@deltas,$queryLen,$refLen);
    coalesceAppal($appal);
    outputAppal($appal,\*STDOUT);
    print "e 0\n";
  }
close(IN);



#------------------------------------------------------------------------
sub promerToAppal
  {
    my ($deltas,$len1,$len2)=@_;
    my $numDeltas=0+@$deltas;
    my (@appal,$x,$y);
    for(my $i=0 ; $i<$numDeltas ; ++$i)
      {
	my $delta=$deltas->[$i];
	my $sign=($delta>0 ? "+" : "-");
	$delta=abs($delta);
	--$delta;
	$delta*=3;
	if($delta>0) {push @appal,"m$delta";$x+=$delta;$y+=$delta}
	if($sign eq "+") {push @appal,"i3";$x+=3}
	else {push @appal,"d3";$y+=3}
      }
    my $dx=$len1-$x; # +1;
    my $dy=$len2-$y; # +1;
    die "$x $len1 $dx , $y $len2 $dy " unless $dx==$dy;
    push @appal,"m$dx";
    return \@appal;
  }
#------------------------------------------------------------------------
sub outputAppal
  {
    my ($appal,$filehandle)=@_;
    my $n=@$appal;
    for(my $i=0 ; $i<$n ; ++$i)
      {
	my $elem=$appal->[$i];
	$elem=~/(.)(.*)/;
	print $filehandle "$1 $2\n";
      }
  }
#------------------------------------------------------------------------
sub coalesceAppal
  {
    my ($appal)=@_;
    my $n=@$appal;
    my $prev=substr($appal->[0],0,1);
    for(my $i=1 ; $i<$n ; ++$i)
      {
	my $this=substr($appal->[$i],0,1);
	if($this eq $prev)
	  {
	    collapseAppal($appal,$i-1,$i);
	    --$n;
	    --$i;
	    next;
	  }
	$prev=$this;
      }
  }
#------------------------------------------------------------------------
sub collapseAppal
  {
    my ($appal,$prevIndex,$thisIndex)=@_;
    my $prev=$appal->[$prevIndex];
    my $this=$appal->[$thisIndex];
    $prev=~/(.)(.*)/;
    my ($prevType,$prevCount)=($1,$2);
    $this=~/(.)(.*)/;
    my ($thisType,$thisCount)=($1,$2);
    die unless $thisType eq $prevType;
    my $combinedCount=$prevCount+$thisCount;
    $appal->[$prevIndex]="$prevType$combinedCount";
    splice(@$appal,$thisIndex,1);
  }
#------------------------------------------------------------------------
#------------------------------------------------------------------------
#------------------------------------------------------------------------
